


# layered_structures_wavelengths.py <job_wavelengths>
# ver 1.1

# P-polarized refelctivity



import numpy as np


import configparser
import sys


import matplotlib.pyplot as plt





commline = sys.argv
print('arguments:',commline)

job = commline[1]


# INI file parsing
Config = configparser.ConfigParser()
Config.read("layered_structures.ini")
print('ini:',Config.sections())

def ConfigSectionMap(section):
    dict1 = {}
    option = Config.options(section)
    for option in options:
        try:
            dict1[option] = Config.get(section, option)
            if dict1[option] == -1:
                DebugPrint("skip: %s" % option)
        except:
            print("exception on %s!" % option)
            dict1[option] = None
    return dict1








def Rinv(tindex,faze):
    pomo = np.array([[np.cos(faze),-1.0j*np.sin(faze)/tindex],[-1.0j*tindex*np.sin(faze),np.cos(faze)]])
    return pomo



def Vzeroinvs(tindex):
    faktor = 1/(2*tindex)
    pomo = np.array( [[faktor*tindex,faktor] , [faktor*tindex,-faktor]] )
    return pomo

def Vzeroinvp(tindex,cosuhel):
    faktor = 1/(2*tindex*cosuhel)
    pomo = np.array( [[faktor*tindex,faktor] , [faktor*tindex,-faktor]] )
    return pomo


def Vfs(tindex):
    pomo = np.array( [[1.0,1.0] , [tindex,-tindex]] )
    return pomo

def Vfp(tindex,cosuhel):
    pomo = np.array( [[cosuhel,cosuhel] , [cosuhel*tindex,-cosuhel*tindex]] )
    return pomo


def tildens(index,cosuhel):
    pomo = index*cosuhel
    return pomo

def tildenp(index,cosuhel):
    pomo = index/cosuhel;
    return pomo


def cosuhelvrstva(dopad,substrat,index):  # COMPLEX ROOTS
    pomo = np.sqrt(1-(substrat/index*np.sin(np.radians(dopad)))**2)
    return pomo


def phase(tloustka,index,vlndelka,cosuhel):
    pomo = 2*3.141592653589*(tloustka/vlndelka)*index*cosuhel
    return pomo









options = Config.options("common")

unit = ConfigSectionMap("common")['units']
optconsts = ConfigSectionMap("common")['optconsts']




options = Config.options(job)

substrate = ConfigSectionMap(job)['substrate']
thicknesses = ConfigSectionMap(job)['thicknesses'].split(',')
materials = ConfigSectionMap(job)['materials'].split(',')
materialu = len(materials)

ambient = ConfigSectionMap(job)['ambient']


wavelengths=ConfigSectionMap(job)['wavelengths'].split(':')
vlndelky = np.arange(int(wavelengths[0]),int(wavelengths[2]),int(wavelengths[1]))
vystup = np.zeros(len(vlndelky))





angles=ConfigSectionMap(job)['angles'].split(':')




path = optconsts+substrate+'.txt'
substrate_data = np.loadtxt(path)
print(path,np.shape(substrate_data))

path = optconsts+materials[0]+'.txt'   # JUST ONE LAYER
materials_data = np.loadtxt(path)
print(path,np.shape(materials_data))

path = optconsts+ambient+'.txt'
ambient_data = np.loadtxt(path)
print(path,np.shape(ambient_data))



# P POLARIZATION

for l in range(len(vlndelky)):

# substrate
   tedlambda = float(vlndelky[l])
   dopaduhel = float(angles[0])                        # JUST ONE ANGLE
   tedcos = complex(np.cos(np.radians(dopaduhel)),0.0   )   
   substrateindex = complex(substrate_data[l][1],substrate_data[l][2])   # JUST ONE WAVELENGTH
   tedtindex = tildenp(substrateindex,tedcos)
   fullmatice = Vzeroinvp(tedtindex,tedcos) 

   print('---------------')
   print('lambda =',tedlambda)
   print('n =',substrateindex)
   print('cos(phi) =',tedcos)
   print('V0invp =',fullmatice)
   print("\n")


# layers

   for k in range(materialu):
   
      tedeps = complex(materials_data[l][1],materials_data[l][2])
      tedindex = complex(np.sqrt((abs(tedeps)+tedeps.real)/2.0),np.sqrt((abs(tedeps)-tedeps.real)/2.0))

      tedcos = cosuhelvrstva(dopaduhel,substrateindex,tedindex)
      tedtindex = tildenp(tedindex,tedcos)
      tedtloustka = float(thicknesses[k])
      tedfaze = phase(tedtloustka,tedindex,tedlambda,tedcos)

      matice = Rinv(tedtindex,tedfaze)

      print('n =',tedindex)
      print('d =',tedtloustka)
      print('cos(phi) =',tedcos)
      print('theta =',tedfaze)
      print('Rinv =',matice)
      print("\n")



      pomomatice = np.dot(fullmatice,matice)
      fullmatice = pomomatice.copy()
   
#   print(fullmatice)


# ambient

   tedindex = complex( ambient_data[l][1],ambient_data[l][2]  )
   tedcos = cosuhelvrstva(dopaduhel,substrateindex,tedindex)
   tedtindex = tildenp(tedindex,tedcos)

   matice = Vfp(tedtindex,tedcos)

   pomomatice = np.dot(fullmatice,matice)
   fullmatice = pomomatice.copy()


   print('n =',tedindex)
   print('cos(phi) =',tedcos)
   print('Vf =',matice)
   print('--------')
   print('P =',fullmatice)






   vystup[l] = abs(fullmatice[1,0]/fullmatice[0,0])**2

   print('|R| =',vystup[l])





np.savetxt(job+'R.txt',list(zip(vlndelky,vystup)))


plt.plot(vlndelky,vystup)
plt.show()

